@@ -1,14 +1,14 @@
 ### Submit/wait for jobs in batch clusters.
 sge.job.finished <- function(jobid)
 {
-  return(system (paste("qstat -j", jobid),
+  return(system (paste0("qstat -j ", jobid),
                  ignore.stdout = TRUE, ignore.stderr = TRUE,
                  intern = FALSE, wait = TRUE))
 }
 
 pbs.job.finished <- function(jobid)
 {
-  return(system (paste("qstat", jobid),
+  return(system (paste0("qstat ", jobid),
                  ignore.stdout = TRUE, ignore.stderr = TRUE,
                  intern = FALSE, wait = TRUE))
 }
@@ -58,7 +58,6 @@ target.runner.qsub <- function(experiment, scenario)
   seed             <- experiment$seed
   configuration    <- experiment$configuration
   instance         <- experiment$instance
-  extra.params     <- experiment$extra.params
   switches         <- experiment$switches
   
   targetRunner <- scenario$targetRunner
@@ -66,7 +65,7 @@ target.runner.qsub <- function(experiment, scenario)
     irace.error ("targetRunner ", shQuote(targetRunner), " cannot be found or is not executable!\n")
   }
 
-  args <- paste(configuration.id, instance.id, seed, instance, extra.params,
+  args <- paste(configuration.id, instance.id, seed, instance,
                 buildCommandLine(configuration, switches))
   output <- runcommand(targetRunner, args, configuration.id, debugLevel)
 
