@@ -65,19 +65,13 @@ sampleUniform <- function (parameters, nbConfigurations, digits,
         # same order as namesParameters, because we sample in the order of the
         # conditions.
         currentType <- parameters$types[[currentParameter]]
+        transform <- parameters$transform[[currentParameter]]
         if (isFixed(currentParameter, parameters)) {
           # We don't even need to sample, there is only one possible value !
           newVal <- get.fixed.value (currentParameter, parameters)
           # The parameter is not a fixed and should be sampled          
-        } else if (currentType == "i") {
-          lowerBound <- as.integer(parameters$domain[[currentParameter]][1])
-          upperBound <- as.integer(parameters$domain[[currentParameter]][2])
-          newVal <- floor(runif(1, min = lowerBound, max = 1 + upperBound))
-        } else if (currentType == "r") {
-          lowerBound <- parameters$domain[[currentParameter]][1]
-          upperBound <- parameters$domain[[currentParameter]][2]
-          newVal <- runif(1, as.double(lowerBound), as.double(upperBound))
-          newVal <- round(newVal, digits)
+        } else if (currentType == "i" || currentType == "r") {
+          newVal <- sample.numerical(currentParameter, parameters, currentType, digits)
         } else if (currentType == "c" || currentType == "o") {
           possibleValues <- parameters$domain[[currentParameter]]
           newVal <- sample(possibleValues, 1)
@@ -141,6 +135,7 @@ sampleModel <- function (parameters, eliteConfigurations, model,
         # should fix this or make it impossible to confuse them.
         currentParameter <- namesParameters[p]
         currentType <- parameters$types[[currentParameter]]
+        transform <- parameters$transform[[currentParameter]]
         if (!conditionsSatisfied(parameters, configuration, currentParameter)) {
           # Some conditions are unsatisfied.
           # Should be useless, NA is ?always? assigned when matrix created
@@ -150,25 +145,19 @@ sampleModel <- function (parameters, eliteConfigurations, model,
           # We don't even need to sample, there is only one possible value !
           newVal <- get.fixed.value (currentParameter, parameters)
           # The parameter is not a fixed and should be sampled
-        } else if (currentType == "i" || currentType == "r") {
-          lowerBound <- paramLowerBound(currentParameter, parameters)
-          upperBound <- paramUpperBound(currentParameter, parameters)
+        } else if (currentType %in% c("i", "r")) {
           mean <- as.numeric(eliteParent[currentParameter])
+          # If there is not value we obtain it from the model
+          if (is.na(mean)) mean <- model[[currentParameter]][[as.character(idEliteParent)]][2]
           if (is.na(mean)) {
             # The elite parent does not have any value for this
             # parameter, let's sample uniformly.
-            newVal <- ifelse(currentType == "i",
-                             floor(runif(1, min = lowerBound, max = 1 + upperBound)),
-                             runif(1, lowerBound, upperBound))
+            ## FIXME: We should save the last used parameter in the model and use it here.
+            newVal <- sample.numerical(currentParameter, parameters, currentType, digits)
           } else {
-            stdDev <- model[[currentParameter]][[as.character(idEliteParent)]]
-            newVal <- ifelse(currentType == "i",
-                             rtnorm(1, mean + 0.5, stdDev, lowerBound, upperBound + 1) - 0.5,
-                             rtnorm(1, mean, stdDev, lowerBound, upperBound))
+            stdDev <- model[[currentParameter]][[as.character(idEliteParent)]][1]
+            newVal <- sample.numerical(currentParameter, parameters, currentType, digits, mean, stdDev)
           }
-          newVal <- ifelse(currentType == "i", round(newVal),
-                           round(newVal, digits))
-          
         } else if (currentType == "o") {
           possibleValues <- paramDomain(currentParameter, parameters)  
           value <- eliteParent[currentParameter]
@@ -176,6 +165,7 @@ sampleModel <- function (parameters, eliteConfigurations, model,
           if (is.na(value)) {
             # The elite parent does not have any value for this
             # parameter, let's sample uniformly
+            ## FIXME: We should save the last used parameter in the model and use it here.
             newVal <- sample(possibleValues, 1)
           } else {
             # Find the position within the vector of possible
@@ -203,7 +193,7 @@ sampleModel <- function (parameters, eliteConfigurations, model,
         configuration[[p]] <- newVal
       }
       
-      configuration <- as.data.frame(configuration, stringsAsFactors=FALSE)
+      configuration <- as.data.frame(configuration, stringsAsFactors = FALSE)
       if (!is.null(repair)) {
         configuration <- repair(configuration, parameters, digits)
       }
@@ -220,3 +210,111 @@ sampleModel <- function (parameters, eliteConfigurations, model,
   }
   return (newConfigurations)
 }
+
+# Sample value for a numerical parameter.
+sample.numerical <- function(param, parameters, type, digits, mean = NULL, stdDev = NULL)
+{
+  lowerBound <- paramLowerBound(param, parameters)
+  upperBound <- paramUpperBound(param, parameters)
+  transform <- parameters$transform[[param]]
+
+  if (transform == "log") {
+    value <- sample.numeric.log(type, lowerBound, upperBound, digits, mean, stdDev)
+  } else {
+    value <- sample.numeric.norm(type, lowerBound, upperBound, digits, mean, stdDev)
+  }
+  return(value)
+}
+
+sample.numeric.norm <- function(type, lowerBound, upperBound, digits, mean, stdDev)
+{
+  if (type == "i") {
+    if (is.null(mean)) {
+      # integer uniform
+      newVal <- floor(runif(1, min = lowerBound, max = 1 + upperBound))
+    } else {
+      # integer from model
+      newVal <- round(rtnorm(1, mean + 0.5, stdDev, lowerBound, upperBound + 1) - 0.5)
+    }
+  } else {
+    irace.assert(type == "r")
+    if (is.null(mean)) {
+      # real uniform
+      newVal <- runif(1, min = as.double(lowerBound), max = as.double(upperBound))
+    } else {
+      # real from model
+      newVal <- round(rtnorm(1, mean, stdDev, lowerBound, upperBound), digits)
+    }
+    newVal <- round(newVal, digits)
+  }
+  return(newVal)
+}
+
+sample.numeric.log <- function(type, lowerBound, upperBound, digits, mean, stdDev)
+{
+  if (is.null(mean)) {
+    trRange <- range.transform.log(lowerBound, upperBound, digits)
+  } else {
+    trRange <- range.transform.log(lowerBound, upperBound, digits, mean)
+  }
+  trLb <- trRange[["trLowerBound"]]
+  trUb <- trRange[["trUpperBound"]]
+  trMean <- trRange[["trMean"]]
+  
+  if (is.null(mean)) {
+    # uniform
+    newVal <- exp(runif(1, min = as.double(trLb), max = as.double(trUb)))
+  } else {
+    # sample from model
+    newVal <- exp(rtnorm(1, trMean, abs(stdDev), trLb, trUb)) 
+  }
+
+  if (type == "i") {
+    newVal <- round(newVal)
+  } else {
+    newVal <- round(newVal, digits)
+  }
+
+  newVal <- check.transform.log(newVal, lowerBound, upperBound)
+  return(newVal)
+}
+
+# Transform the range [LB,UB] into [log(LB),log(UB)].
+# It needs to pay attention to shift the range in the
+# positive domain if LB <= 0. The mean is also transformed
+# (if provided, otherwise it can be ignored).
+# A similar check will be required after applying the
+# transformation (see function check.transform.log() ).
+range.transform.log <- function(lowerBound, upperBound, digits, mean = 1)
+{
+  # cannot compute log(0)
+  if (lowerBound <= 0) {
+    trLowerBound <- -digits
+    trUpperBound <- log(upperBound - lowerBound)
+    trMean <- log(mean + lowerBound)
+  } else {
+    trLowerBound <- log(lowerBound)
+    trUpperBound <- log(upperBound)
+    trMean <- log(mean)
+  }
+  return(list(trLowerBound = trLowerBound,
+              trUpperBound = trUpperBound,
+              trMean = trMean))
+}
+
+# Adjust the sampled value if the lower bound is <=0, and
+# check that it does not fall outside the allowed range.
+check.transform.log <- function(newVal, lowerBound, upperBound)
+{
+  # check if LB was negative, then readjust
+  if (lowerBound <= 0) {
+    newVal <- newVal + lowerBound
+  }
+  # never too sure
+  if (newVal < lowerBound) {
+    newVal <- lowerBound
+  } else if (newVal > upperBound) {
+    newVal <- upperBound
+  }
+  return(newVal)
+}
