@@ -1,21 +1,3 @@
-# FIXME: It may be faster to create a single expression that concatenates all
-# the elements of forbidden using '|'
-checkForbidden <- function(configurations, forbidden)
-{
-  # We have to use a variable name that will never appear in
-  # configurations, so .FORBIDDEN .
-  for (.FORBIDDEN in forbidden) {
-    #print(.FORBIDDEN)
-    configurations <- subset(configurations, eval(.FORBIDDEN))
-    #print(configurations)
-    #print(str(configurations))
-    ## FIXME: This is normally called with a single configuration. Thus, it
-    ## would be faster to break as soon as nrow(configurations) < 1
-  }
-  #print(nrow(configurations))
-  return(configurations)
-}
-
 # Sets irace variables from a recovery file.  It is executed in the
 # parent environment which must be irace().
 #
@@ -112,7 +94,7 @@ numeric.configurations.equal <- function(x, configurations, parameters, threshol
 similarConfigurations <- function(configurations, parameters, threshold)
 {
   debug.level <- getOption(".irace.debug.level", 0)
-  if (debug.level >= 1) cat ("# Computing similarity of configurations .")
+  if (debug.level >= 1) irace.note ("Computing similarity of configurations .")
 
   # Create vectors of categorical and numerical
   p <- parameters$types %in% c("c","o")
@@ -347,6 +329,14 @@ startParallel <- function(scenario)
       requireNamespace("parallel", quietly = TRUE)
       if (.Platform$OS.type == 'windows') {
         .irace$cluster <- parallel::makeCluster(parallel)
+        # In Windows, this needs to be exported, or we get:
+        ## Error in checkForRemoteErrors(val) : 
+        ##  2 nodes produced errors; first error: could not find function "target.runner"
+        parallel::clusterExport(.irace$cluster, list("target.runner"), envir=.irace)
+        # In addition, we export the global environment because the user may
+        # have defined stuff there. There must be a better way to do this, but
+        # I cannot figure it out. R sucks sometimes.
+        parallel::clusterExport(.irace$cluster, ls(envir=.GlobalEnv))
       }
     }
   }
@@ -412,45 +402,192 @@ addInstances <- function(scenario, instancesList, n.instances)
 do.experiments <- function(configurations, ninstances, scenario, parameters)
 {
   output <- lapply(1:ninstances, race.wrapper, configurations = configurations, 
+                   bounds = if (scenario$capping) rep(scenario$boundMax, nrow(configurations))
+                            else NULL,
                    which.alive = 1:nrow(configurations), which.exe = 1:nrow(configurations), 
                    parameters = parameters, scenario = scenario)
                                         
   Results <- matrix(nrow = ninstances, ncol = nrow(configurations),
                     dimnames = list(1:ninstances, as.character(configurations[, ".ID."])))
-  experimentLog <- matrix(nrow = 0, ncol = 3,
-                          dimnames = list(NULL, c("instance", "configuration", "time")))
+  experimentLog <- matrix(nrow = 0, ncol = 4,
+                          dimnames = list(NULL, c("instance", "configuration", "time", "bound")))
                           
   # Extract results
-  ## FIXME: There must be a faster way to do this.
   for (j in 1:ninstances) {
-    for (i in 1:nrow(configurations)) {
-      Results[j, i] <- output[[j]][[i]]$cost
-      output.time   <- output[[j]][[i]]$time
-      irace.assert(!is.null(output.time))
-      ## FIXME: It would be more efficient to build three vectors and cbind them once as:
-      # experimentLog <- cbind(instance = rep(1:ninstances, nrow(configurations)), configuration = configurations[, ".ID."], time = output.time)
-      experimentLog <- rbind(experimentLog,
-                             c(j, configurations[i, ".ID."], output.time))
-    }
+    vcost <- unlist(lapply(output[[j]], "[[", "cost"))
+    if (scenario$capping)
+      vcost <- applyPAR(vcost, scenario)
+    Results[j, ] <- vcost
+    vtimes <- unlist(lapply(output[[j]], "[[", "time"))
+    irace.assert(!any(is.null(vtimes)))
+    experimentLog <- rbind(experimentLog,
+                           cbind(j, configurations$.ID., vtimes, 
+                                 if (scenario$capping) scenario$boundMax else NA))
   }
   
-  return (list(experiments = Results, experimentLog = experimentLog))
+  rejectedIDs <- configurations[apply(is.infinite(Results), 2, any), ".ID."]
+  return (list(experiments = Results, experimentLog = experimentLog, rejectedIDs = rejectedIDs))
 }
 
-## FIXME: Move this to the irace.Rd file
-#' High-level function to use iterated Race
+## Gets the elite configurations time matrix from the experiment log
+generateTimeMatrix <- function(elites, experimentLog)
+{
+  is.elite <- experimentLog[,"configuration"] %in% elites$.ID.
+  # Remove everything that we don't need.
+  experimentLog <- experimentLog[is.elite, c("configuration", "instance", "time", "bound"), drop = FALSE]
+  experimentLog[, "time"] <- pmin(experimentLog[,"time"], experimentLog[, "bound"])
+  # FIXME: It would be better to use spread() from tidyr
+  resultsTime <- reshape(as.data.frame(experimentLog), direction = "wide",
+                         idvar = "instance", timevar = "configuration",
+                         drop = "bound")
+  rownames(resultsTime) <- resultsTime$instance
+  resultsTime <- resultsTime[order(resultsTime$instance), , drop = FALSE]
+  colnames(resultsTime) <- substring(colnames(resultsTime), nchar("time.") + 1)
+  resultsTime <- as.matrix(resultsTime[, as.character(elites$.ID.), drop = FALSE])
+  return(resultsTime)           
+}
+
+allConfigurationsInit <- function(scenario, parameters, debugLevel)
+{
+  if (!is.null.or.empty(scenario$configurationsFile)) {
+    allConfigurations <- readConfigurationsFile(scenario$configurationsFile,
+                                                parameters, debugLevel)
+    allConfigurations <- cbind(.ID. = 1:nrow(allConfigurations),
+                               allConfigurations,
+                               .PARENT. = NA)
+    rownames(allConfigurations) <- allConfigurations$.ID.
+    num <- nrow(allConfigurations)
+    allConfigurations <- checkForbidden(allConfigurations, scenario$forbiddenExps)
+    if (nrow(allConfigurations) < num) {
+      cat("# Warning: some of the configurations in the configurations file were forbidden",
+          "and, thus, discarded\n")
+    }
+    cat("# Adding", nrow(allConfigurations), "initial configuration(s) from file",
+        shQuote(scenario$configurationsFile), "\n")
+  } else {
+    configurations.colnames <- c(".ID.", names(parameters$conditions), ".PARENT.")
+    allConfigurations <-
+      as.data.frame(matrix(ncol = length(configurations.colnames),
+                           nrow = 0,
+                           dimnames = list(NULL, configurations.colnames)))
+  }
+  return(allConfigurations)
+}
+
+#' irace
+#'
+#' \code{irace} implements iterated Race. It receives some parameters to be tuned 
+#'   and returns the best configurations found, namely, the elite configurations 
+#'   obtained from the last iterations (and sorted by rank).
 #' 
-#' This function implement iterated Race. It receives some parameters to be tuned and returns the best
-#' configurations found, namely, the elite configurations obtained from the last iterations (and sorted by rank).
+#' @param scenario Data structure containing \pkg{irace} settings.The data structure 
+#'   has to be the one returned by the function \code{\link{defaultScenario}} and 
+#'   \code{\link{readScenario}}.
+#' @param parameters Data structure containing the parameter definition. The data 
+#'   structure has to be the one returned by the function \code{\link{readParameters}}.
 #'
-#' @param parameter data-structure containing the parameter definition. The data-structure has to be the one
-#' returned by the function \code{readParameters()}. See documentation of this function for details.
+#' @details The function \code{irace} executes the tuning procedure using 
+#'  the information provided in \code{scenario} and \code{parameters}. Initially it checks 
+#'  the correctness of \code{scenario} and recovers a previous execution if 
+#'  \code{scenario$recoveryFile} is set. A R data file log of the execution is created 
+#'  in \code{scenario$logFile}.
 #'
-#' @param scenario data-structure containing irace settings.The data-structure has to be the one
-#' returned by the function \code{readScenario()}. See documentation of this function for details.
-#' @return Elites configurations obtained after the last iteration
-#' @callGraphPrimitives
-#' @note This is a note for the function \code{iteratedRace}
+#' @return A data frame with the set of best algorithm configurations found by \pkg{irace}. 
+#'  The data frame has the following columns:
+#'  \describe{
+#'    \item{\code{.ID.}}{Internal id of the candidate configuration.}
+#'    \item{\code{Parameter names}}{One column per parameter name in \code{parameters}.}
+#'    \item{\code{.PARENT.}}{Internal id of the parent candidate configuration.}
+#'  }
+#'  Additinally, this function saves an R data file containing an object called
+#'  \code{iraceResults}. The path of the file is indicated in \code{scenario$logFile}. 
+#'  The \code{iraceResults} object is a list with the following structure:
+#'  
+#' \describe{
+#'
+#'     \item{\code{scenario}}{The scenario R object containing the \pkg{irace}
+#'     options used for the execution. See \code{\link{defaultScenario}} help
+#'     for more information.}
+#'  
+#'     \item{\code{parameters}}{The parameters R object containing the
+#'     description of the target algorithm parameters. See
+#'     \code{\link{readParameters}}.}
+#'  
+#'     \item{\code{allConfigurations}}{The target algorithm configurations
+#'     generated by \pkg{irace}. This object is a data frame, each row is a
+#'     candidate configuration, the first column (\code{.ID.}) indicates the
+#'     internal identifier of the configuration, the following columns
+#'     correspond to the parameter values, each column named as the parameter
+#'     name specified in the parameter object. The final column
+#'     (\code{.PARENT.})  is the identifier of the configuration from which
+#'     model the actual configuration was sampled.}
+#'   
+#'     \item{\code{allElites}}{A list that contains one element per iteration,
+#'     each element contains the internal identifier of the elite candidate
+#'     configurations of the corresponding iteration (identifiers correspond to
+#'     \code{allConfigurations$.ID.}).}
+#'     
+#'     \item{\code{iterationElites}}{A vector containing the best candidate
+#'     configuration internal identifier of each iteration. The best
+#'     configuration found corresponds to the last one of this vector.}
+#'     
+#'     \item{\code{experiments}}{A matrix with configurations as columns and
+#'     instances as rows. Column names correspond to the internal identifier of
+#'     the configuration (\code{allConfigurations$.ID.}).}
+#'     
+#'     \item{\code{experimentLog}}{A matrix with columns \code{iteration,
+#'     instance, configuration, time}.  This matrix contains the log of all the
+#'     experiments that \pkg{irace} performs during its execution.  The
+#'     instance column refers to the index of the \code{scenario$instancesList}
+#'     data frame. Time is saved ONLY when reported by the targetRunner.}
+#'
+#'     \item{\code{softRestart}}{A logical vector that indicates if a soft
+#'     restart was performed on each iteration. If \code{FALSE}, then no soft
+#'     restart was performed.}
+#'
+#'     \item{\code{state}}{A list that contains the state of \pkg{irace}, the
+#'     recovery is done using the information contained in this object.}
+#'     
+#'     \item{\code{testing}}{A list that constains the testing results. The
+#'     elements of this list are: \code{experiments} a matrix with the testing
+#'     expriments of the selected configurations in the same format as the
+#'     explained above and \code{seeds} a vector with the seeds used to execute
+#'     each experiment.}
+#'
+#' }
+#'
+#' @examples
+#' \dontrun{
+#' parameters <- readParameters("parameters.txt")
+#' scenario <- readScenario(filename = "scenario.txt",
+#'                          scenario = defaultScenario())
+#' irace(scenario = scenario, parameters = parameters)
+#' }
+#'
+#' @seealso
+#'  \describe{
+#'  \item{\code{\link{irace.main}}}{a higher-level command-line interface to \code{irace}.}
+#'  \item{\code{\link{readScenario}}}{for reading a configuration scenario from a file.}
+#'  \item{\code{\link{readParameters}}}{read the target algorithm parameters from a file.}
+#'  \item{\code{\link{defaultScenario}}}{returns the default scenario settings of \pkg{irace}.}
+#'  \item{\code{\link{checkScenario}}}{to check that the scenario is valid.}
+#' }
+#' 
+#' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @export
+## FIXME: Move this to the irace.Rd file
+# High-level function to use iterated Race
+# 
+# This function implement iterated Race. It receives some parameters to be tuned and returns the best
+# configurations found, namely, the elite configurations obtained from the last iterations (and sorted by rank).
+#
+# @param parameter data-structure containing the parameter definition. The data-structure has to be the one
+# returned by the function \code{readParameters()}. See documentation of this function for details.
+#
+# @param scenario data-structure containing irace settings.The data-structure has to be the one
+# returned by the function \code{readScenario()}. See documentation of this function for details.
+# @return Elites configurations obtained after the last iteration
+# @note This is a note for the function \code{iteratedRace}
 irace <- function(scenario, parameters)
 {
   catInfo <- function(..., verbose = TRUE) {
@@ -469,18 +606,21 @@ irace <- function(scenario, parameters)
   }
   
   scenario <- checkScenario(defaultScenario(scenario))
-  
   # Recover state from file?
   if (!is.null(scenario$recoveryFile)) {
     irace.note ("Resuming from file: '", scenario$recoveryFile,"'\n")
     recoverFromFile(scenario$recoveryFile)
     # We call checkScenario again to fix any inconsistencies in the recovered
     # data.
+    firstRace <- FALSE
     scenario <- checkScenario(scenario)
     startParallel(scenario)
-    on.exit(stopParallel())
+    on.exit(stopParallel(), add = TRUE)
+
   } else { # Do not recover
+    firstRace <- TRUE
     scenario <- irace.init (scenario)
+    forbiddenExps <- scenario$forbiddenExps
     debugLevel <- scenario$debugLevel
     # Set options controlling debug level.
     # FIXME: This should be the other way around, the options set the debugLevel.
@@ -488,44 +628,20 @@ irace <- function(scenario, parameters)
     options(.irace.debug.level = debugLevel)
     
     # Create a data frame of all configurations ever generated.
-    namesParameters <- names(parameters$conditions)
-
-    nbUserConfigurations <- 0
-    if (!is.null.or.empty(scenario$configurationsFile)) {
-      allConfigurations <- readConfigurationsFile(scenario$configurationsFile,
-                                          parameters, debugLevel)
-      allConfigurations <- cbind(.ID. = 1:nrow(allConfigurations),
-                             allConfigurations,
-                             .PARENT. = NA)
-      rownames(allConfigurations) <- allConfigurations$.ID.
-      num <- nrow(allConfigurations)
-      allConfigurations <- checkForbidden(allConfigurations, scenario$forbiddenExps)
-      if (nrow(allConfigurations) < num) {
-        cat("# Warning: some of the configurations in the configurations file were forbidden",
-            "and, thus, discarded\n")
-      }
-      cat("# Adding", nrow(allConfigurations), "initial configuration(s) from file",
-          shQuote(scenario$configurationsFile), "\n")
-      nbUserConfigurations <- nrow(allConfigurations)
-    } else {
-      configurations.colnames <- c(".ID.", namesParameters, ".PARENT.")
-      allConfigurations <-
-        as.data.frame(matrix(ncol = length(configurations.colnames),
-                             nrow = 0,
-                             dimnames = list(NULL, configurations.colnames)))
-    }
+    allConfigurations <- allConfigurationsInit(scenario, parameters, debugLevel)
+    nbUserConfigurations <- nrow(allConfigurations)
+  
     # To save the logs
-    iraceResults <- list()
-    iraceResults$scenario <- scenario
-    iraceResults$irace.version <- irace.version
-    iraceResults$parameters <- parameters
-    iraceResults$iterationElites <- NULL
-    iraceResults$allElites <- list()
-    iraceResults$experiments <- matrix(nrow = 0, ncol = 0)
-    iraceResults$experimentLog <- matrix(nrow = 0, ncol = 4,
-                                         dimnames = list(NULL,
-                                           c("iteration", "instance", "configuration", "time")))
-
+    iraceResults <- list(
+      scenario = scenario,
+      irace.version = irace.version,
+      parameters = parameters,
+      allElites = list(),
+      experiments = matrix(nrow = 0, ncol = 0),
+      experimentLog = matrix(nrow = 0, ncol = 5,
+                              dimnames = list(NULL,
+                                              c("iteration", "instance", "configuration", "time", "bound")))
+    )
     model <- NULL
     nbConfigurations <- 0
     eliteConfigurations <- data.frame()
@@ -551,9 +667,10 @@ irace <- function(scenario, parameters)
     experimentsUsedSoFar <- 0
     timeUsed <- 0
     timeEstimate <- NA 
+    rejectedIDs <- c()
 
     startParallel(scenario)
-    on.exit(stopParallel())                                          
+    on.exit(stopParallel(), add = TRUE)
  
     if (scenario$maxTime == 0) {
       remainingBudget <- scenario$maxExperiments
@@ -565,11 +682,12 @@ irace <- function(scenario, parameters)
       ## discarding these configurations.
       ninstances <- scenario$firstTest
       estimationTime <- ceiling(scenario$maxTime * scenario$budgetEstimation)
+      timeEstimate <- scenario$boundMax
       irace.note("Estimating execution time using ", 100 * scenario$budgetEstimation,
                  "% of ", scenario$maxTime, " = ", estimationTime, "\n")
 
       # Estimate the number of configurations to be used
-      nconfigurations <- max(2, floor(scenario$parallel/ninstances))
+      nconfigurations <- max(2, floor(scenario$parallel / ninstances))
       
       next.configuration <- 1
       while (TRUE) {
@@ -578,7 +696,7 @@ irace <- function(scenario, parameters)
           newConfigurations <- sampleUniform(parameters,
                                              nconfigurations - nrow(allConfigurations),
                                              digits = scenario$digits,
-                                             forbidden = scenario$forbiddenExps,
+                                             forbidden = forbiddenExps,
                                              repair = scenario$repairConfiguration)
           newConfigurations <-
             cbind (.ID. = max(0, allConfigurations$.ID.) + 1:nrow(newConfigurations),
@@ -590,18 +708,30 @@ irace <- function(scenario, parameters)
         # Execute tests
         output <- do.experiments(configurations = allConfigurations[next.configuration:nconfigurations, ],
                                  ninstances = ninstances, scenario = scenario, parameters = parameters)  
-
         iraceResults$experimentLog <- rbind(iraceResults$experimentLog,
                                             cbind(rep(0, nrow(output$experimentLog)),
                                                   output$experimentLog)) 
-
-        timeUsed     <- sum(timeUsed, output$experimentLog[, "time"], na.rm = TRUE)
-        timeEstimate <- mean(iraceResults$experimentLog[, "time"], na.rm = TRUE)   
         
         iraceResults$experiments <- merge.matrix (iraceResults$experiments,
                                                   output$experiments)
         rownames(iraceResults$experiments) <- 1:nrow(iraceResults$experiments)
-        
+        rejectedIDs <- c(rejectedIDs, output$rejectedIDs)
+        iraceResults$rejectedConfigurations <- rejectedIDs
+        forbiddenExps <- c(forbiddenExps,
+                           buildForbiddenExp(configurations = allConfigurations[
+                                               allConfigurations$.ID. %in% output$rejectedIDs, , drop = FALSE],
+                                             parameters = parameters))
+                
+        # For the used time, we count the time reported in all configurations
+        # including rejected ones. 
+        timeUsed <- sum(timeUsed, output$experimentLog[, "time"], na.rm = TRUE)
+        # In the rare case all configurations are eliminated, we keep timeEstimate as 
+        # initially defined (boundMax)
+        if (nrow(allConfigurations) > length(rejectedIDs)) {
+          timeEstimate <- mean(iraceResults$experimentLog[
+                               iraceResults$experimentLog[, "configuration"] %!in% rejectedIDs, "time"], 
+                               na.rm = TRUE)
+        }
         next.configuration <- nconfigurations + 1
         
         # Calculate how many new configurations:
@@ -611,22 +741,27 @@ irace <- function(scenario, parameters)
         # that we can execute in parallel.
         new.conf <- min(new.conf, max(1, floor(scenario$parallel / ninstances)))
 
-        if (timeUsed >= estimationTime || new.conf == 0) {
+        if (timeUsed >= estimationTime || new.conf == 0 || nconfigurations == 1024) {
           break
         } else {
-          nconfigurations <- nconfigurations + new.conf
+          nconfigurations <- min(1024, nconfigurations + new.conf)
         }
       }
+      
+      if (length(rejectedIDs) > 0) {
+        irace.note ("Immediately rejected configurations: ",
+                    paste0(rejectedIDs, collapse = ", ") , "\n")
+      }
   
       irace.note("Estimated execution time is ", timeEstimate, " based on ",
                  next.configuration - 1, " configurations and ",
-                 ninstances," instances. Used time: ", timeUsed, " .\n")
+                 ninstances," instances. Used time: ", timeUsed, "\n")
       
       # Update budget
       remainingBudget <- round((scenario$maxTime - timeUsed) / timeEstimate)
 
       experimentsUsedSoFar <- experimentsUsedSoFar + nrow(iraceResults$experimentLog)
-      eliteConfigurations <- allConfigurations[1:(next.configuration - 1),]
+      eliteConfigurations <- allConfigurations[allConfigurations$.ID. %!in% rejectedIDs, ,drop = FALSE]
 
       # Without elitist, the racing does not re-use the results computed during
       # the estimation.  This means that the time used during estimation needs
@@ -646,21 +781,21 @@ irace <- function(scenario, parameters)
               scenario$nbExperimentsPerIteration)
 
     # Check that the budget is enough, for the time estimation case we reduce
-    # the number of iterations..
+    # the number of iterations.
     repeat {
       if (scenario$maxTime == 0
           || checkMinimumBudget (remainingBudget, minSurvival, nbIterations,
                                  scenario = scenario)) {
         break;
       }
-      irace.note("Warning:",
+      irace.note("\nWarning:",
                  " with the current settings and estimated time per run,",
                  " irace will not have enough budget to execute the minimum",
                  " number of iterations.",
                  " Execution will continue by assuming that the estimated time",
                  " is too high and reducing the minimum number of iterations,",
                  " however, if the estimation was correct or too low,",
-                 " results might not be better than random sampling.")
+                 " results might not be better than random sampling.\n")
       nbIterations <- nbIterations - 1
     }
   } #end of do not recover
@@ -681,8 +816,15 @@ irace <- function(scenario, parameters)
           else paste0("# time budget: ", scenario$maxTime - timeUsed, "\n"),
           "# mu: ", max(scenario$mu, scenario$firstTest), "\n",
           "# deterministic: ", scenario$deterministic, "\n",
+          if (scenario$capping) 
+            paste0("# capping: ", scenario$cappingType, "\n",
+                   "# type bound: ", scenario$boundType, "\n", 
+                   "# maximum bound: ", scenario$boundMax, "\n", 
+                   "# par bound: ", scenario$boundPar, "\n", 
+                   "# bound digits: ", scenario$boundDigits, "\n"),
           verbose = FALSE)
 
+  
   while (TRUE) {
     # Recovery info 
     iraceResults$state <- list(.Random.seed = get(".Random.seed", .GlobalEnv),
@@ -698,7 +840,9 @@ irace <- function(scenario, parameters)
                                nbIterations = nbIterations,
                                remainingBudget = remainingBudget,
                                timeUsed = timeUsed,
-                               timeEstimate = timeEstimate)
+                               timeEstimate = timeEstimate,
+                               rejectedIDs = rejectedIDs,
+                               forbiddenExps = forbiddenExps)
     # Consistency checks
     irace.assert(sum(!is.na(iraceResults$experiments)) == experimentsUsedSoFar)
     irace.assert(nrow(iraceResults$experimentLog) == experimentsUsedSoFar)
@@ -739,7 +883,7 @@ irace <- function(scenario, parameters)
               scenario$nbExperimentsPerIteration)
     
     # Compute the number of configurations for this race.
-    if (scenario$elitist && indexIteration > 1) {
+    if (scenario$elitist && !firstRace) {
       nOldInstances <- nrow(iraceResults$experiments)
       nbConfigurations <-
         computeNbConfigurations(currentBudget, indexIteration,
@@ -756,7 +900,7 @@ irace <- function(scenario, parameters)
                             nElites = 0, nOldInstances = 0,
                             newInstances = 0)
     }
-
+    
     # If a value was given as a parameter, then this value limits the maximum,
     # but if we have budget only for less than this, then we have run out of
     # budget.
@@ -765,15 +909,17 @@ irace <- function(scenario, parameters)
         nbConfigurations <- scenario$nbConfigurations
       } else if (currentBudget < remainingBudget) {
         # We skip one iteration
+        catInfo("Not enough budget for this iteration, ",
+                "skipping to the next one.")
         indexIteration <- indexIteration + 1
         next
       } else {
         catInfo("Stopped because ",
-                "there is not enough budget to enforce the value of nbConfigurations")
+                "there is not enough budget to enforce the value of nbConfigurations.")
         return (eliteConfigurations)
       }
     }
-
+    
     # Stop if the number of configurations to test is NOT larger than the minimum.
     if (nbConfigurations <= minSurvival) {
       catInfo("Stopped because there is not enough budget left to race more than ",
@@ -782,15 +928,24 @@ irace <- function(scenario, parameters)
       return (eliteConfigurations)
     }
 
-    # Stop if  the number of configurations to produce is not greater than
-    # the number of elites.
-    if (nbConfigurations <= nrow(eliteConfigurations)) {
+
+    # If we have too many eliteConfigurations, reduce their number. This can
+    # happen before the first race due to the initial budget estimation.
+    if (firstRace) {
+      if (nbConfigurations < nrow(eliteConfigurations)) {
+        eliteRanks <- overall.ranks(iraceResults$experiments, stat.test = scenario$testType)
+        eliteConfigurations <- eliteConfigurations[order(eliteRanks), ]
+        eliteConfigurations <- eliteConfigurations[1:nbConfigurations, ]
+      }
+    } else if (nbConfigurations <= nrow(eliteConfigurations)) {
+      # Stop if  the number of configurations to produce is not greater than
+      # the number of elites.
       catInfo("Stopped because ",
               "there is not enough budget left to race newly sampled configurations")
       #(number of elites  + 1) * (mu + min(5, indexIteration)) > remainingBudget" 
       return (eliteConfigurations)
     }
-
+    
     if (scenario$elitist) {
       # The non-elite have to run up to the first test. The elites consume
       # budget at most up to the new instances.
@@ -818,9 +973,9 @@ irace <- function(scenario, parameters)
             
     iraceResults$softRestart[indexIteration] <- FALSE
     # Sample for the first time.
-    if (indexIteration == 1) {
+    if (firstRace) {
       # If we need more configurations, sample uniformly.
-      nbNewConfigurations <- nbConfigurations - nrow(allConfigurations)
+      nbNewConfigurations <- nbConfigurations - sum(allConfigurations$.ID. %!in% rejectedIDs)
       if (nbNewConfigurations > 0) {
         # Sample new configurations.
         if (debugLevel >= 1) {
@@ -829,32 +984,32 @@ irace <- function(scenario, parameters)
         }
         newConfigurations <- sampleUniform(parameters, nbNewConfigurations,
                                            digits = scenario$digits,
-                                           forbidden = scenario$forbiddenExps,
+                                           forbidden = forbiddenExps,
                                            repair = scenario$repairConfiguration)
         newConfigurations <-
           cbind (.ID. = max(0, allConfigurations$.ID.) + 1:nrow(newConfigurations),
                  newConfigurations)
         allConfigurations <- rbind(allConfigurations, newConfigurations)
         rownames(allConfigurations) <- allConfigurations$.ID.
+        raceConfigurations <- allConfigurations[allConfigurations$.ID. %!in% rejectedIDs, , drop = FALSE]
       } else if (nbNewConfigurations < 0) {
-        # We let the user know that not all configurations will be used
+        # We let the user know that not all configurations will be used.
         if (nbUserConfigurations > nbConfigurations) {
           catInfo("Only ", nbConfigurations,
-                  " from the configurations file will be used",
+                  " from the initial configurations will be used",
                   verbose = FALSE)
-          # LESLIE: should we remove them in this case??
-          # if (scenario$maxTime <= 0 && nbUserConfigurations > nbConfigurations)
-          #   allConfigurations <- allConfigurations[1:nbConfigurations,]
         }
         
-        # This is made only in case that the number of configurations used in the
-        # time estimation is more than needed.
-        if (nrow(eliteConfigurations) > nbConfigurations) {
-          eliteConfigurations <- eliteConfigurations[1:nbConfigurations, ]
+        # This is made only in case that the number of configurations used in
+        # the time estimation is more than needed.
+        if (nrow(eliteConfigurations) == nbConfigurations) {
+          raceConfigurations <- eliteConfigurations
+        } else {
+          raceConfigurations <- allConfigurations[allConfigurations$.ID. %!in% rejectedIDs, , drop = FALSE]
+          raceConfigurations <- raceConfigurations[1:nbConfigurations,]
         }
-        
-      }
-      raceConfigurations <- allConfigurations[1:nbConfigurations,]
+      } # end of indexIteration == 1
+      
     } else {
       # How many new configurations should be sampled?
       nbNewConfigurations <- nbConfigurations - nrow(eliteConfigurations)
@@ -868,31 +1023,29 @@ irace <- function(scenario, parameters)
       if (debugLevel >= 1) {
         irace.note("Sample ", nbNewConfigurations, " configurations from model\n")
       }
-
-      #cat("# ", format(Sys.time(), usetz=TRUE), " sampleModel()\n")
       newConfigurations <- sampleModel(parameters, eliteConfigurations,
-                                   model, nbNewConfigurations,
-                                   digits = scenario$digits,
-                                   forbidden = scenario$forbiddenExps,
-                                   repair = scenario$repairConfiguration)
-      #cat("# ", format(Sys.time(), usetz=TRUE), " sampleModel() DONE\n")
+                                       model, nbNewConfigurations,
+                                       digits = scenario$digits,
+                                       forbidden = forbiddenExps,
+                                       repair = scenario$repairConfiguration)
+
       # Set ID of the new configurations.
       newConfigurations <- cbind (.ID. = max(0, allConfigurations$.ID.) +
-                              1:nrow(newConfigurations), newConfigurations)
-      raceConfigurations <- rbind(eliteConfigurations[, 1:ncol(allConfigurations)],
-                              newConfigurations)
+                                    1:nrow(newConfigurations), newConfigurations)
+      raceConfigurations <- rbind(eliteConfigurations[, colnames(newConfigurations)],
+                                  newConfigurations)
       rownames(raceConfigurations) <- raceConfigurations$.ID.
 
       if (scenario$softRestart) {
         #          Rprof("profile.out")
         tmp.ids <- similarConfigurations (raceConfigurations, parameters,
-                                      threshold = scenario$softRestartThreshold)
+                                          threshold = scenario$softRestartThreshold)
         #          Rprof(NULL)
         if (!is.null(tmp.ids)) {
           if (debugLevel >= 1)
             irace.note("Soft restart: ", paste(collapse = " ", tmp.ids), " !\n")
           model <- restartConfigurations (raceConfigurations, tmp.ids, model,
-                                      parameters, nbNewConfigurations)
+                                          parameters, nbNewConfigurations, scenario$digits)
           iraceResults$softRestart[indexIteration] <- TRUE
           iraceResults$model$afterSR[[indexIteration]] <- model
           if (debugLevel >= 2) { printModel (model) }
@@ -901,13 +1054,13 @@ irace <- function(scenario, parameters)
           newConfigurations <- sampleModel(parameters, eliteConfigurations,
                                            model, nbNewConfigurations,
                                            digits = scenario$digits,
-                                           forbidden = scenario$forbiddenExps,
+                                           forbidden = forbiddenExps,
                                            repair = scenario$repairConfiguration)
           #cat("# ", format(Sys.time(), usetz=TRUE), " sampleModel() DONE\n")
           # Set ID of the new configurations.
           newConfigurations <- cbind (.ID. = max(0, allConfigurations$.ID.) + 
                                   1:nrow(newConfigurations), newConfigurations)
-          raceConfigurations <- rbind(eliteConfigurations[, 1:ncol(allConfigurations)],
+          raceConfigurations <- rbind(eliteConfigurations[, colnames(newConfigurations)],
                                       newConfigurations)
           rownames(raceConfigurations) <- raceConfigurations$.ID.
         }
@@ -927,7 +1080,11 @@ irace <- function(scenario, parameters)
     # Get data from previous elite tests 
     elite.data <- NULL
     if (scenario$elitist && nrow(eliteConfigurations) > 0) {
-      elite.data <- iraceResults$experiments[, as.character(eliteConfigurations[,".ID."]), drop=FALSE]
+      elite.data <- list()
+      elite.data[["experiments"]] <- iraceResults$experiments[, as.character(eliteConfigurations[,".ID."]), drop=FALSE]
+      if (scenario$capping)
+        elite.data[["time"]] <- generateTimeMatrix(elites = eliteConfigurations, 
+                                                   experimentLog = iraceResults$experimentLog)
     }
     
     .irace$next.instance <- max(nrow(iraceResults$experiments), 0) + 1
@@ -947,10 +1104,8 @@ irace <- function(scenario, parameters)
                          maxExp = currentBudget, 
                          minSurvival = minSurvival,
                          elite.data = elite.data,
-                         elitistNewInstances = if (indexIteration > 1)
-                                                 scenario$elitistNewInstances
-                                               else 0)
-
+                         elitistNewInstances = if (firstRace) 0
+                                               else scenario$elitistNewInstances)
     # Update experiments
     # LESLIE: Maybe we can think is make iraceResults an environment, so these values
     # can be updated in the race function.
@@ -960,15 +1115,27 @@ irace <- function(scenario, parameters)
                                         cbind(rep(indexIteration, nrow(raceResults$experimentLog)),
                                               raceResults$experimentLog))
     
-    # Merge new results
+    # Merge new results.
     iraceResults$experiments <- merge.matrix (iraceResults$experiments,
                                               raceResults$experiments)
-    
+                                             
+    if (length(raceResults$rejectedIDs) > 0) {
+      rejectedIDs <- c(rejectedIDs, raceResults$rejectedIDs)
+      iraceResults$rejectedConfigurations <- rejectedIDs
+      forbiddenExps <- c(forbiddenExps,
+                         buildForbiddenExp(
+                           configurations = allConfigurations[
+                             allConfigurations$.ID. %in% raceResults$rejectedIDs, , drop = FALSE],
+                           parameters = parameters))
+    }
+
     experimentsUsedSoFar <- experimentsUsedSoFar + raceResults$experimentsUsed
-    # Update remaining budget
+    # Update remaining budget.
     if (scenario$maxTime > 0) { 
-      timeUsed <- sum (timeUsed, raceResults$experimentLog[,"time"], na.rm=TRUE)
-      timeEstimate <- mean(iraceResults$experimentLog[,"time"], na.rm=TRUE)
+      timeUsed <- sum (timeUsed, raceResults$experimentLog[,"time"], na.rm=TRUE)                                   
+      timeEstimate <- mean(iraceResults$experimentLog[
+                                          iraceResults$experimentLog[, "configuration"] %!in% rejectedIDs, "time"], 
+                           na.rm=TRUE)
       remainingBudget <- round((scenario$maxTime - timeUsed) / timeEstimate)
     } else {
       remainingBudget <- remainingBudget - raceResults$experimentsUsed
@@ -994,9 +1161,9 @@ irace <- function(scenario, parameters)
     iraceResults$iterationElites <- c(iraceResults$iterationElites, eliteConfigurations$.ID.[1])
     iraceResults$allElites[[indexIteration]] <- eliteConfigurations$.ID.
     
-    if (indexIteration == 1) {
+    if (firstRace) {
       if (debugLevel >= 1)  { irace.note("Initialise model\n") }
-      model <- initialiseModel(parameters, eliteConfigurations)
+      model <- initialiseModel(parameters, eliteConfigurations, scenario$digits)
     }
       
     if (debugLevel >= 1) {
@@ -1009,6 +1176,7 @@ irace <- function(scenario, parameters)
     }
 
     indexIteration <- indexIteration + 1
+    firstRace <- FALSE
     if (scenario$debugLevel >= 3) {
       irace.note ("Memory used in irace():\n")
       irace.print.memUsed()
